
;**************************************************************************
;*                    I2C Master Test Board                               *
;**************************************************************************
;**************************************************************************
;*          v 1.04 - 31.10.2013                                           *
;*                                                                        *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F886 used                                        *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************



;------------------------------------------------------
;device addresses:
;	EEPROM:			1010 000 x
;	I2c 2 USB Bridge	0110 000 x
;------------------------------------------------------



;	SDA -- SDA (RC4)
;	SCL -- SCL (RC3)


 INCLUDE "P16F886.INC"


_U001			     EQU     B'1110000011000100'	;see Page 206 of Datasheet
_U002			     EQU     B'1111111111111111'	;see Page 207 of Datasheet

;	__CONFIG	_CONFIG1, _DEBUG_OFF & _PLL_EN & _BORV_2_5 & _BOR_OFF & _CP_OFF & _MCLR_DIS & _PWRT_DIS & _WDT_OFF & _INTRC_OSC_NOCLKOUT
;	__CONFIG	_CONFIG2, _VCAP_DIS


	__CONFIG	_CONFIG1, _U001

	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

;interrupt
W_TEMP		EQU	H'22'
STATUS_TEMP	EQU	H'23'
PCLATH_TEMP	EQU	H'24'
;---------------------------------------------------------------------
;---------------------------------------------------------------------
kbd_buffer	EQU	H'25'
v_tmp2		EQU	H'26'
v_ad0		EQU	H'27'		;EEPROM address - lowest 8 bits
v_ad1		EQU	H'28'		;EEPROM address - highest bits

;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		nop
PAGE0
;
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01011111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		bcf	OPTION_REG,T0CS	;set-up TIMER0 as timer
		bsf	OPTION_REG,PSA	;prescaler NOT for TIMER0
		;
		CLRF	INTCON		;Disable interrupts
		clrf	IOCB

		MOVLW	B'11000000'	;PORTA
		MOVWF	TRISA

		MOVLW	B'10111111'
		MOVWF	TRISB

		MOVLW	B'00011000'	;PORTC
		MOVWF	TRISC

		MOVLW	B'11111000'	;PORTE = output; RE3 = input
		MOVWF	TRISE

		movlw	B'10000000'
		movwf	WPUB

		bsf	STATUS,RP0
		bsf	STATUS,RP1
		clrf	ANSEL
		clrf	ANSELH

		bcf	STATUS,RP0	;Switch Back to reg. Bank 0
		bcf	STATUS,RP1

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;set up I2C BUS for communication as a master
		movlw	B'00101000'		;see Datasheet Page 177
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSPCON

;set-up I2C speed
		bsf	STATUS,RP0
		movlw	B'00111111'
		movwf	SSPADD
		clrf	SSPSTAT
		bcf	STATUS,RP0
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------


; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       initialization  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

init_main	clrf	PORTC
		clrf	PORTA
		clrf	PORTB

;
		clrf	PIE1
		clrf	PIE2

		bcf	INTCON,GIE		;disable all interrupts


main_loop	bsf	PORTB,D'006'
		call	d2
		btfss	PORTB,D'007'
		goto	send_data
		bcf	PORTB,D'006'
		call	d2
		btfss	PORTB,D'007'
		goto	send_data
		goto	main_loop



send_data	bsf	PORTB,D'006'
		call	demo_ctd3p3

		goto	main_loop	


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
i2c_start	bsf	STATUS,RP0
		bsf	SSPCON2, SEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	STATUS,RP0
		bsf	SSPCON2, PEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	STATUS,RP0
		bcf	SSPCON2, ACKDT		;ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	STATUS,RP0
		bsf	SSPCON2, ACKDT		;NOT ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	STATUS,RP0
		bsf	SSPCON2, RCEN
		bcf	STATUS,RP0
		call	d55
		movf	SSPBUF,0
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	movwf	SSPBUF
		call	d55		;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	STATUS,RP0
		bsf	SSPCON2, RSEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------


;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------

;---------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------

i2c2usb		nop
		call	i2c_send
		call	dr56
		return
;-------------------------------------
;-------------------------------------
demo_ctd3p3	nop

		bcf	PORTA,D'000'		;A0 = 0
		bcf	PORTA,D'001'		;A1 = 0
		bcf	PORTA,D'002'		;A2 = 0

		clrf	v_ad0
		movlw	D'016'
		movwf	v_tmp2

		call	i2c_start

		movlw	B'01100000'		;I2C address of USB bridge
		call	i2c2usb

		movlw	'E'
		call	i2c2usb
		movlw	'E'
		call	i2c2usb
		movlw	'P'
		call	i2c2usb
		movlw	'R'
		call	i2c2usb
		movlw	'O'
		call	i2c2usb
		movlw	'M'
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	'2'
		call	i2c2usb
		movlw	'4'
		call	i2c2usb
		movlw	'L'
		call	i2c2usb
		movlw	'C'
		call	i2c2usb
		movlw	'x'
		call	i2c2usb
		movlw	'x'
		call	i2c2usb
		movlw	':'
		call	i2c2usb

		movlw	D'013'			;ENTER
		call	i2c2usb

		movlw	D'013'			;ENTER
		call	i2c2usb


		call	i2c_stop
;-------------------------------------------------------------
;EEPROM data read
eep_rn2		call	i2c_start

		movlw	B'10100000'
		call	i2c_send

		movf	v_ad0,0
		call	i2c_send

		call	i2c_start

		movlw	B'10100001'
		call	i2c_send

		call	i2c_receive
		movwf	kbd_buffer

		call	i2c_not_ack

		call	i2c_stop
;-------------------------------------------------------------
		call	i2c_start

		movlw	B'01100000'		;I2C address of USB bridge
		call	i2c2usb

		call	write_hex2

		movlw	'-'

		decfsz	v_tmp2,1
		goto	dalej1
;new line
		movlw	D'016'
		movwf	v_tmp2
		movlw	D'013'			;ENTER

dalej1		nop
		call	i2c2usb

		call	i2c_stop

		incf	v_ad0,1
		btfss	STATUS,Z
		goto	eep_rn2
;-------------------------------------------------------------
		call	i2c_start

		movlw	B'01100000'		;I2C address of USB bridge
		call	i2c2usb

		movlw	D'013'			;ENTER
		call	i2c2usb

		movlw	'T'
		call	i2c2usb
		movlw	'r'
		call	i2c2usb
		movlw	'a'
		call	i2c2usb
		movlw	'n'
		call	i2c2usb
		movlw	's'
		call	i2c2usb
		movlw	'm'
		call	i2c2usb
		movlw	'i'
		call	i2c2usb
		movlw	's'
		call	i2c2usb
		movlw	's'
		call	i2c2usb
		movlw	'i'
		call	i2c2usb
		movlw	'o'
		call	i2c2usb
		movlw	'n'
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	'f'
		call	i2c2usb
		movlw	'i'
		call	i2c2usb
		movlw	'n'
		call	i2c2usb
		movlw	'i'
		call	i2c2usb
		movlw	's'
		call	i2c2usb
		movlw	'h'
		call	i2c2usb
		movlw	'e'
		call	i2c2usb
		movlw	'd'
		call	i2c2usb
		movlw	D'013'			;ENTER
		call	i2c2usb

		movlw	D'013'			;ENTER
		call	i2c2usb


		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex2	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9x

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_Ax

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_Bx

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_Cx

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_Dx

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_Ex

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_Fx
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9x

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_Ax

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_Bx

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_Cx

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_Dx

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_Ex

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_Fx


		return
;---------------------------------------------------------------------------------
wm_0x		movlw	'0'
		call	i2c2usb
		return
;-------------------------------
wm_1x		movlw	'1'
		call	i2c2usb
		return
;-------------------------------
wm_2x		movlw	'2'
		call	i2c2usb
		return
;-------------------------------
wm_3x		movlw	'3'
		call	i2c2usb
		return
;-------------------------------
wm_4x		movlw	'4'
		call	i2c2usb
		return
;-------------------------------
wm_5x		movlw	'5'
		call	i2c2usb
		return
;-------------------------------
wm_6x		movlw	'6'
		call	i2c2usb
		return
;-------------------------------
wm_7x		movlw	'7'
		call	i2c2usb
		return
;-------------------------------
wm_8x		movlw	'8'
		call	i2c2usb
		return
;-------------------------------
wm_9x		movlw	'9'
		call	i2c2usb
		return
;-------------------------------
wm_Ax		movlw	'A'
		call	i2c2usb
		return
;-------------------------------
wm_Bx		movlw	'B'
		call	i2c2usb
		return
;-------------------------------
wm_Cx		movlw	'C'
		call	i2c2usb
		return
;-------------------------------
wm_Dx		movlw	'D'
		call	i2c2usb
		return
;-------------------------------
wm_Ex		movlw	'E'
		call	i2c2usb
		return
;-------------------------------
wm_Fx		movlw	'F'
		call	i2c2usb
		return
;-------------------------------




;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;		*********************************************
;		*               delay routines              *
;		*********************************************
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------
dr56		MOVLW   D'010'
		MOVWF   TIMER2
DEL_LOOPb1q	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPb2q	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPb2q
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPb1q
		return


;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END